//+------------------------------------------------------------------+
//|                                        wpr + bollinger bands.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DeepSkyBlue
#property indicator_color2  Snow
#property indicator_color3  DarkOrange
#property indicator_color4  Snow
#property indicator_style2  STYLE_DASH
#property indicator_style3  STYLE_DASH
#property indicator_style4  STYLE_DASH
#property indicator_width1  4
#property indicator_maximum 120
#property indicator_minimum -20
#property indicator_level1  20
#property indicator_level2  80
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor Magenta

//
//
//
//
//

extern string TimeFrame        = "Current time frame";
extern int    HLR_Range        = 50;
extern int    Length           = 25;
extern double Speed            = 8;
extern bool   Adaptive         = true;
extern int    BandsPeriod      = 30;
extern double BandsDeviations  = 2.2;

extern bool   ShowArrows       = true;
extern string ArrowsIdentifier = "roma";
extern color  ArrowUpColor     = Aqua;
extern color  ArrowDownColor   = Red;  
extern bool   Interpolate      = true;

extern string  _               = "alerts settings";
extern bool    alertsOn        = true;
extern bool    alertsOnCurrent = true;
extern bool    alertsMessage   = true;
extern bool    alertsSound     = false;
extern bool    alertsEmail     = false;


//
//
//
//
//

double HLR[];
double bandsUp[];
double bandsMi[];
double bandsDo[];
double trend[];
double atrend[];
double stored[][21];


string IndicatorFileName;
bool   CalculatingHLR;
bool   ReturningBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,HLR);
   SetIndexBuffer(1,bandsUp);
   SetIndexBuffer(2,bandsMi);
   SetIndexBuffer(3,bandsDo);
   SetIndexBuffer(4,trend);
   SetIndexBuffer(5,atrend);
   
   Length = MathMax(Length,   1);
   Speed  = MathMax(Speed ,-1.5);

   //
   //
   //
   //
   //
   
   IndicatorFileName   = WindowExpertName();
   CalculatingHLR      = (TimeFrame=="CalculateHLR"); if (CalculatingHLR) return(0);
   ReturningBars       = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
   timeFrame           = stringToTimeFrame(TimeFrame);
         
   //
   //
   //
   //
   //
               
   IndicatorShortName("Range oma with bands "+timeFrameToString(timeFrame)+" ("+HLR_Range+","+BandsPeriod+","+DoubleToStr(BandsDeviations,2)+")");
   return(0);
}

int deinit()
{
   if (ShowArrows)
   {
      int compareLength = StringLen(ArrowsIdentifier);
      for (int i=ObjectsTotal(); i>= 0; i--)
      {
      string name = ObjectName(i);
      if (StringSubstr(name,0,compareLength) == ArrowsIdentifier)
       ObjectDelete(name);  
      }
   }
   return(0);
}




int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { HLR[0] = limit+1; return(0); }
           if (ArrayRange(stored,0) != Bars) ArrayResize(stored,Bars);

   
   
      if (CalculatingHLR || timeFrame==Period())
      {
          
      for(i=limit, r=Bars-limit-1; i>=0; i--,r++)
      {
      double hhv  = iAverage(High[iHighest(NULL, 0, MODE_HIGH, HLR_Range, i)],Length,Speed,Adaptive,r,0);
      double llv  = iAverage(Low[iLowest(NULL, 0,   MODE_LOW,  HLR_Range, i)],Length,Speed,Adaptive,r,7);
      double m_pr = iAverage((High[i] + Low[i]) / 2.0,                        Length,Speed,Adaptive,r,14);
      if(hhv!=llv)
      HLR[i] = 100.0 * (m_pr-llv)/(hhv-llv); 
      else HLR[i] = 0.0; 
      }
      for(i=limit; i>=0; i--) 
      {
      double deviations = iStdDevOnArray(HLR,0,BandsPeriod,0,MODE_LWMA,i);
      bandsMi[i] = iMAOnArray(HLR,0,BandsPeriod,0,MODE_LWMA,i);
      bandsUp[i] = bandsMi[i]+deviations*BandsDeviations;
      bandsDo[i] = bandsMi[i]-deviations*BandsDeviations;
      
      
      trend[i] = trend[i+1];
      if (HLR[i] < bandsDo[i] && HLR[i+1] >= bandsDo[i+1] ) trend[i] =  1;
      if (HLR[i] > bandsUp[i] && HLR[i+1] <= bandsUp[i+1] ) trend[i] = -1;
       
       manageArrow(i);  
      }
      manageAlerts(); 
      return(0);
   }

   
      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"","returnBars",0,0)*timeFrame/Period()));
      for(i=limit; i>=0; i--)
      {
        int y = iBarShift(NULL,timeFrame,Time[i]);
        HLR[i]     = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,Length,Speed,Adaptive,BandsPeriod,BandsDeviations,0,y);
        bandsUp[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,Length,Speed,Adaptive,BandsPeriod,BandsDeviations,1,y);
        bandsMi[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,Length,Speed,Adaptive,BandsPeriod,BandsDeviations,2,y);
        bandsDo[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,Length,Speed,Adaptive,BandsPeriod,BandsDeviations,3,y);
        trend[i]   = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,Length,Speed,Adaptive,BandsPeriod,BandsDeviations,4,y);
        manageArrow(i);

         
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         

         datetime time = iTime(NULL,timeFrame,y);
         for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
         double factor = 1.0 / n;
         for(int k = 1; k < n; k++)
         {
  	      HLR[i+k]     = k*factor*HLR[i+n]     + (1.0-k*factor)*HLR[i];
  	      bandsUp[i+k] = k*factor*bandsUp[i+n] + (1.0-k*factor)*bandsUp[i];
  	      bandsMi[i+k] = k*factor*bandsMi[i+n] + (1.0-k*factor)*bandsMi[i];
  	      bandsDo[i+k] = k*factor*bandsDo[i+n] + (1.0-k*factor)*bandsDo[i];
        }  	            
   }
      
   manageAlerts();
      
   return(0);
}



void manageArrow(int i)
{
   if (ShowArrows && !CalculatingHLR)
   {
      double dist = iATR(NULL,0,20,i)/2.0;
      ObjectDelete(ArrowsIdentifier+Time[0]);         
            
      //
      //
      //
      //
      //
           
      atrend[i] = atrend[i+1];
         if (HLR[i] < bandsDo[i] && HLR[i+1] >= bandsDo[i+1] ) atrend[i] =  1;
         if (HLR[i] > bandsUp[i] && HLR[i+1] <= bandsUp[i+1] ) atrend[i] = -1;
         if (trend[i] !=trend[i+1])
         {
            string name = ArrowsIdentifier+Time[i];
            if (atrend[i] == 1)
            {
               ObjectCreate(name,OBJ_ARROW,0, Time[i],Low[i]-dist );
                  ObjectSet(name,OBJPROP_ARROWCODE,241);
                  ObjectSet(name,OBJPROP_COLOR,ArrowUpColor);
            }
            else
            {
               ObjectCreate(name,OBJ_ARROW,0, Time[i],High[i]+dist );
                  ObjectSet(name,OBJPROP_ARROWCODE,242);
                  ObjectSet(name,OBJPROP_COLOR,ArrowDownColor);
            }
         }
   }
}

void manageAlerts()
{
   if (!CalculatingHLR && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" Range osc trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Range osc "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}


#define E1  0
#define E2  1
#define E3  2
#define E4  3
#define E5  4
#define E6  5
#define res 6

//
//
//
//
//

double iAverage(double price, double averagePeriod, double const, bool adaptive, int r, int ashift=0)
{
   double e1=stored[r-1][E1+ashift];  double e2=stored[r-1][E2+ashift];
   double e3=stored[r-1][E3+ashift];  double e4=stored[r-1][E4+ashift];
   double e5=stored[r-1][E5+ashift];  double e6=stored[r-1][E6+ashift];

   //
   //
   //
   //
   //

      if (adaptive && (averagePeriod > 1))
      {
         double minPeriod = averagePeriod/2.0;
         double maxPeriod = minPeriod*5.0;
         int    endPeriod = MathCeil(maxPeriod);
         double signal    = MathAbs((price-stored[r-endPeriod][res]));
         double noise     = 0.00000000001;

            for(int k=1; k<endPeriod; k++) noise=noise+MathAbs(price-stored[r-k][res]);

         averagePeriod = ((signal/noise)*(maxPeriod-minPeriod))+minPeriod;
      }
      
      //
      //
      //
      //
      //
      
      double alpha = (2.0+const)/(1.0+const+averagePeriod);

      e1 = e1 + alpha*(price-e1); e2 = e2 + alpha*(e1-e2); double v1 = 1.5 * e1 - 0.5 * e2;
      e3 = e3 + alpha*(v1   -e3); e4 = e4 + alpha*(e3-e4); double v2 = 1.5 * e3 - 0.5 * e4;
      e5 = e5 + alpha*(v2   -e5); e6 = e6 + alpha*(e5-e6); double v3 = 1.5 * e5 - 0.5 * e6;

   //
   //
   //
   //
   //

   stored[r][E1+ashift]  = e1;  stored[r][E2+ashift] = e2;
   stored[r][E3+ashift]  = e3;  stored[r][E4+ashift] = e4;
   stored[r][E5+ashift]  = e5;  stored[r][E6+ashift] = e6;
   stored[r][res+ashift] = price;
   return(v3);
}

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}